import es
import gamethread
import time

info = es.AddonInfo()
info.name = 'DM EndRound'
info.version = "2.4"
info.basename = 'dmendround'
info.description = ('Forces "Round Draw" when the round time has elapsed, '
    'meant to be used on DeathMatch servers where rounds normally last '
    'infinitely')
info.url = 'http://addons.eventscripts.com/addons/view/dmendround/'

# server variable stuff...
version_variable = es.ServerVar('dm_endround_v', info.version,
    '%s version number' % info.name)
version_variable.set(info.version)
version_variable.makepublic()

dm_realround = es.ServerVar('dm_realround', 0,
    'Boolean value telling whether a round is active')
isenabled = es.ServerVar('dm_endround', 1, 'Is %s enabled' % info.name)
dm_endround_effect = es.ServerVar('dm_endround_effect', 1,
    'Play sound and display message or not?')
disable_on_maps = es.ServerVar('dm_disabled_maps', 0,
    'Disable %s on specified maps, multiple names must be separated with '
    'commas (,); names are considered to be either full or partial map names'
    % info.name)
dm_endround_draw_time = es.ServerVar('dm_endround_draw_time', 2,
    'Freeze time in seconds after round draw when dm_endround_effect is 1')
dm_endround_draw_msg = es.ServerVar('dm_endround_draw_msg', 'Round Draw!',
    'Message to display to players as endround effect.')
dm_endround_draw_sound = es.ServerVar('dm_endround_draw_sound',
    'radio/rounddraw.wav',
    'Sound file to play on round draw, path relative to sound directory.')
dm_endround_warning_time = es.ServerVar('dm_endround_warning_time', 0,
    'Time in seconds before round ending to show a round-end warning message. '
    '0 disables.')
dm_endround_warning_msg = es.ServerVar('dm_endround_warning_msg', 0,
    'Message to display to players on warning time.')
dm_endround_warning_sound = es.ServerVar('dm_endround_warning_sound', 0,
    'Sound file to play on warning, path relative to sound directory.')
# NOTE: ES BUG, cannot use empty default value for cvars, have to use 0

mapname = es.ServerVar('eventscripts_currentmap')
# end of server variable stuff...

mp_roundtime = es.ServerVar('mp_roundtime')
roundtime = float(mp_roundtime) * 60.0
last_dm_roundtime = 0
warned = False

laststart = time.time()

def _ticker():
    now = time.time()
    if now-laststart >= roundtime:
        round_draw()
    elif not warned and now-laststart >= roundtime - float(
            dm_endround_warning_time):
        warn_round_end()

def _start_ticking():
    global isticking
    if not isticking:
        es.addons.registerTickListener(_ticker)
        isticking = True

def _stop_ticking():
    global isticking
    if isticking:
        es.addons.unregisterTickListener(_ticker)
        isticking = False
    
isticking = False
    
def load():
    if not es.exists('command', 'dm_roundtime'):
        es.regcmd('dm_roundtime', '%s/dm_roundtime' % info.basename,
            'Overrides mp_roundtime for ' + info.name)
    if es.exists('variable', 'endround'):
        es.flags('remove', 'cheat', 'endround')
    add_downloadables()

def add_downloadables():
    sound = str(dm_endround_draw_sound)
    if sound and sound != '0':
        es.stringtable('downloadables', 'sound/'+sound)
    sound = str(dm_endround_warning_sound)
    if sound and sound != '0':
        es.stringtable('downloadables', 'sound/'+sound)

def unload():
    if isticking:
        es.addons.unregisterTickListener(_ticker)

def dm_roundtime():
    global roundtime, last_dm_roundtime
    argc = es.getargc()
    if argc == 1:
        mins, secs = divmod(last_dm_roundtime, 60)
        es.dbgmsg(0, '"dm_roundtime" = "%d:%02d" min. %0.6f' % (
            mins, secs, 0.0,
        ))
        es.dbgmsg(0, ' - If nonzero, overrides mp_roundtime for the value %s '
            'uses for round length' % (
            info.name)
        )
        es.dbgmsg(0, ' - Can be either integer, floating point number or '
            'string of form "min:sec"')
        mins, secs = divmod(roundtime, 60)
        es.dbgmsg(0, ' - Currently effective roundtime = %d:%02d' % (
            mins, secs,
        ))
        return
    elif argc == 2:
        timearg = es.getargv(1)
        try:
            mins = float(timearg)
            secs = 0.0
        except ValueError:
            try:
                mins, secs = (float(x) for x in timearg.split(':', 1))
            except ValueError:
                es.dbgmsg(0, 'Invalid value %s for dm_roundtime' % timearg)
                return
    elif argc == 4:
        if es.getargv(2) == ':':
            try:
                mins = float(es.getargv(1))
                secs = float(es.getargv(3))
            except ValueError:
                es.dbgmsg(0, 'Invalid value %s for dm_roundtime' %
                    es.getargs()
                )
                return
    else:
        es.dbgmsg(0, 'Invalid value %s for dm_roundtime' % es.getargs())
        return
    
    secs = int(secs + mins * 60)
    if secs <= 0:
        secs = 0
        roundtime = float(mp_roundtime) * 60.0
    else:
        roundtime = float(secs)
    
    last_dm_roundtime = secs

def round_draw():
    _stop_ticking()
    # Check for maps exclusions
    curmap = str(mapname)
    mapfilter = str(disable_on_maps)
    if mapfilter and mapfilter != '0':
        for mapmatch in mapfilter.split(','):
            if mapmatch and mapmatch in curmap:
                # disabled on this map
                es.dbgmsg(1,
                    '%s: Map name "%s" matched disable filter "%s",'
                    ' not forcing round draw.' % (
                        info.name, curmap, mapmatch,
                    )
                )
                return
    
    if int(isenabled):
        dm_realround.set(0)
        if int(dm_endround_effect):
            for userid in es.getUseridList():
                es.playsound(userid, str(dm_endround_draw_sound), 1.0)
                # the following only works in CS:S, but who cares...
                es.setplayerprop(userid, 'CCSPlayer.m_iPlayerState', 1)
            es.centermsg(str(dm_endround_draw_msg))
            invoke_round_draw(float(dm_endround_draw_time))
        else:
            invoke_round_draw(0.0)
    else:
        es.dbgmsg(1,
            '%s: CVAR dm_endround is set to 0, not forcing round draw.' % (
                info.name,
            )
        )

def invoke_round_draw(delay):
    if es.exists('variable', 'endround'):
        es.delayed(delay, 'endround')
    elif es.exists('variable', 'mp_restartround'): # TF2
        es.ServerCommand('mp_restartround %f' % (delay or 0.1))

def warn_round_end():
    global warned
    warned = True
    msg = str(dm_endround_warning_msg)
    if msg and msg != '0':
        es.centermsg(msg)
    sound = str(dm_endround_warning_sound)
    if sound and sound != '0':
        for userid in es.getUseridList():
            es.playsound(userid, sound, 1.0)

# ALL THE RANDOM EVENTS:
def es_map_start(ev):
    add_downloadables()

# ROUND_START
def round_start(ev):
    global laststart, roundtime, warned
    laststart = time.time()
    _start_ticking()
    if last_dm_roundtime == 0:
        roundtime = float(mp_roundtime)*60.0
    warned = False

def round_freeze_end(ev):
    round_start(ev)
    dm_realround.set(1)

def game_start(ev):
    round_start(ev)

def dod_round_start(ev):
    round_start(ev)

def dod_round_active(ev):
    round_start(ev)
    dm_realround.set(1)

def teamplay_round_start(ev):
    round_start(ev)

def teamplay_round_active(ev):
    round_start(ev)
    dm_realround.set(1)

# ROUND_END
def round_end(ev):
    dm_realround.set(0)
    _stop_ticking()

def game_end(ev):
    round_end(ev)

def dod_game_over(ev):
    round_end(ev)

def dod_round_win(ev):
    round_end(ev)

def tf_game_over(ev):
    round_end(ev)

def teamplay_round_win(ev):
    round_end(ev)

def teamplay_game_over(ev):
    round_end(ev)
